/**
 * BaseVideoRequest.js, 2013
 * @author Chornous Igor ichornous@heliostech.hk
 */

/**
 * @namespace provs
 */
var provs = (function(provs) {

// {{{ BaseVideoRequest class
/**
 * @class BaseVideoRequest
 * @implements IVideoRequest
 */
provs.BaseVideoRequest = function()
{
    var ctx = this;

    // {{{ Properties
    /**
     * @name requestUri
     * @type nsIURI
     */
    var requestUri = null;

    /**
     * @name requestOrigin
     * @type Document
     */
    var requestOrigin = null;

    /**
     * @name nameHint
     * @type String
     */
    var nameHint = "";

    /**
     * @name contentType
     * @type String
     */
    var contentType = "video/x-flv";

    /**
     * @name contentLength
     * @type Number
     */
    var contentLength = -1;

    /**
     * @name comparator
     * @type Function
     */
    var comparator = provs.BaseVideoRequest.defaultComparator;
    // }}}

    // {{{ init method
    /**
     * @member init
     * @param {nsIURI} uri
     * @param {Document} document
     */
    this.init = function(params)
    {
        requestUri = params.uri;
        requestOrigin = params.document;
        nameHint = params.name
            ? params.name
            : AntLib.safeGet(requestOrigin, "title");

        if (params.contentType)
            contentType = params.contentType;
        if (params.contentLength)
            contentLength = params.contentLength;
    };
    // }}}

    // {{{ setName method
    /**
     * @member setName
     * @param {String} hint
     * @returns {undefined} nothing
     */
    this.setName = function(hint)
    {
        nameHint = hint;
    };
    // }}}

    /**
     * IVideoRequest implementation
     */
    // {{{ release method
    /**
     * @member release
     * @returns {undefined} nothing
     */
    this.release = function()
    {
        requestUri = null;
        requestOrigin = null;
    };
    // }}}

    // {{{ compare method
    /**
     * @member compare
     * @param {IVideoRequest} request
     * @returns {Boolean}
     */
    this.compare = function(request)
    {
        return comparator(ctx, request);
    };
    // }}}

    // {{{ getContentType method
    /**
     * @member getContentType
     * @returns {String}
     */
    this.getContentType = function()
    {
        return contentType;
    };
    // }}}

    // {{{ getContentLength method
    /**
     * @member getContentLength
     * @returns {Number}
     */
    this.getContentLength = function()
    {
        return contentLength;
    };
    // }}}

    // {{{ getName method
    /**
     * @member getName
     * @returns {String}
     */
    this.getName = function()
    {
        return nameHint;
    };
    // }}}

    // {{{ getOrigin method
    /**
     * Returns a document object associated with the request.
     *
     * @member getOrigin
     * @returns {Document}
     */
    this.getOrigin = function()
    {
        return requestOrigin;
    };
    // }}}

    // {{{ getUri method
    /**
     * @member getUri
     * @returns {nsIURI}
     */
    this.getUri = function()
    {
        return requestUri;
    };
    // }}}
};

// }}}

/**
 * Misc
 */

// {{{ defaultComparator private method
/**
 * @private
 * @member defaultComparator
 * @param {BaseVideoRequest} a
 * @param {BaseVideoRequest} b
 * @returns {Boolean}
 */
provs.BaseVideoRequest.defaultComparator = function(a, b)
{
    return a.getUri().spec == b.getUri().spec;
};
// }}}

return provs;
})({});
