//
//  queryobserver.js
//  firefox
//
//  Created by Zak on 2008-06-09.
//  Contributor Brian King
//  Contributor Chornous Igor
//  Copyright 2008-2012 Ant.com. All rights reserved.
//

// {{{ AntQueryObserver class
/**
 * Observer called when firefox download something
 * @class AntQueryObserver
 * @constructor No parameters
 */
function AntQueryObserver()
{
    const Ci = Components.interfaces;
    const Cc = Components.classes;

    // {{{ Search strategies
    /**
     * DefaultSearchStrategy must remain the last one in the list
     *
     * @name searchStrategy
     * @type Array<ISearchStrategy>
     */
    var searchStrategies = [
        new provs.YtSearchStrategy()
        , new provs.DmSearchStrategy()
        , new provs.DefaultSearchStrategy()
    ];
    // }}}

    // {{{ observe method
    /**
     * The function called by firefox, entry point of this class
     * @param request       The object containing the request params
     * @param topic         The specific event that trigger the function
     * @param data          the request data (seems to be empty)
     */
    this.observe = function(aRequest, topic, data)
    {
        if (topic.substring(0, 16) != "http-on-examine-")
            return;

        if (!gBrowser.getBrowserForDocument)
            return;

        var httpChannel = aRequest.QueryInterface(Ci.nsIHttpChannel);
        if ((httpChannel.responseStatus < 200)
            || (httpChannel.responseStatus > 299))
            return;

        var document = AntLib.getDocumentByRequest(httpChannel);
        if (!document)
            return;

        // Ensure that the request has an associated browser
        // In some cases firefox may prefetch the page
        if (!gBrowser.getBrowserForDocument(document))
            return;

        var strategy = getSearchStrategy(document, httpChannel);
        if (!strategy)
            return;

        try
        {
            strategy.search(
                document
                , httpChannel
                , AntGrabber.foundFlvLink);
        }
        catch (e)
        {
            AntLib.toLog("Unexpected program failure"
                         + "\nError: " + e
                         + "\nFrame:\n" + e.stack);
        }
    };
    // }}}

    // {{{ getSearchStrategy method
    /**
     * Looks up for an appropriate detecting strategy
     *
     * @private
     * @member getSearchStrategy
     * @param {Document} document Owning document
     * @param {nsIHttpChannel} channel
     * @returns {ISearchStrategy}
     */
    var getSearchStrategy = function(document, channel)
    {
        for each (var strategy in searchStrategies)
        {
            if (strategy.isApplicable(document, channel))
                return strategy;
        }

        return null;
    };
    // }}}
};
// }}}
