// 
//  antrpc.js
//  firefox
//  
//  Created by Dima Sidorchenko on 2008-10-23.
//  Copyright 2008-2012 Ant.com. All rights reserved.
//


var AntRPC = {
    reportsServer : 'http://rpc.ant.com/',
    rank : function(url, doc) {
        
        try {
            
            if ( AntLib.inPrivate )
                return;
            
            var self = AntRPC;
            var ref = doc.referrer;
            
            
            var request = {
                "version":  "1.0",
                "id":       1,
                "method":   "rank",
                "params":
                [
                    {
                        "url":      encodeURI(url),
                        "ref":      encodeURI(ref),
                        "uid":      AntRank.UUID,
                        "uagent":   AntPrefs.getUserAgent(),
                        "lang":     AntPrefs.getAcceptLang()
                    }
                ],
                "agent":     AntBar.getAgent()
            };
            
            if (AntLib.getDomain(url) != AntLib.getDomain(ref)) {
                AntRank.setLabelRank(false, doc);
            }
            
            AntLib.makeRequest( request, 
                                self.reportsServer,
                                function  (httpRequest) {
                                    
                                    var rank;
                                    try {
                                      rank = JSON.parse(httpRequest.responseText).result;
                                    } 
                                    catch(e) {
                                      AntLib.toLog(e);
                                      rank = 'NA';
                                    }
                                    
                                    doc.__antrank__ = rank;
                                    AntRank.setLabelRank(rank, doc);
                                }
                              );
        }
        catch(e) {
            AntLib.toLog("AntRPC.rank : " + e);
        }
    },
    
    installCallback: function( state, addonList ) {
        
        try {
            
            var self = AntRPC;
            var report = {
                "version":  "1.0",
                "id":       1,
                "method":   "log_deploy",
                "params":
                [
                    "ant.report.addon." + state,
                    {
                        "uagent":   AntPrefs.getUserAgent(),
                        "locale":   AntPrefs.getAcceptLang(),
                        "uid":      AntRank.UUID,
                        "exit":     "success"
                    }
                ],
                "agent":    AntBar.getAgent()
            };
            
            //for uninstall/upgrade
            if ( state != 'install' ) {
              report.params[1].install_ts = AntPrefs.installTs;
              report.params[1].addon_list = AntLib.getShortExtList(addonList);
            }
            
            if ( state == 'upgrade' ) {
              report.params[1].prev_agent = AntPrefs.prevAgent;
            }
            
            var callback = null;
            if ( state != 'uninstall' ) {
                callback = function(httpRequest) {
                                
                                try {
                                    
                                    if ( AntPrefs.installTs == 0 ) {
                                        var jsonRes = JSON.parse(httpRequest.responseText);
                                        AntPrefs.installTs = jsonRes.result.server_ts;
                                    }
                                }
                                catch(e) {
                                  AntLib.toLog(e);
                                }
                            };
            }
            
            AntLib.makeRequest( report, self.reportsServer, callback );
        }
        catch (e) {
             AntLib.toLog("AntRPC.install : " + e);
        }
    },
    install : function(state) {
        
        if ( AntLib.inPrivate )
            return;
        
        var self = AntRPC;
        var addonList;
        
        if ( AntLib.getFirefoxVersion() < 4 ) {
            
            self.installCallback( state );
        }
        else {
            Components.utils.import("resource://gre/modules/AddonManager.jsm");
            AddonManager.getAllAddons( function(addonList){
                                        self.installCallback( state, addonList );
                                       } );
        }
    },
    
    downloadCallback: function(flvlink, addonList) {
        try {
            if (!AntPrefs.canSendStats) {
                AntLib.toLog("Denied by user");
                return;
            }
            
            var self = AntRPC;
            var report = {
                "version":  "1.0",
                "id":       1,
                "method":   "log",
                "params":
                [
                    "ant.report.addon.download",
                    {
                        "url":      encodeURI(gBrowser.selectedBrowser.contentWindow.location.href),
                        "ref":      encodeURI(gBrowser.contentDocument.referrer),
                        "streams":  [],
                        "uid":      AntRank.UUID,
                        "uagent":   AntPrefs.getUserAgent()
                    }
                ],
                "agent":    AntBar.getAgent()
            };
            if (flvlink) {
                report.params[1].streams.push( {
                    "stream":  encodeURI(flvlink.videoUrl),
                    "type":    flvlink.contentType,
                    "length":  flvlink.contentLength
                });
            }
            else {
                report.params[1].addon_list = AntLib.getShortExtList(addonList);
            }
            
            AntLib.makeRequest(report, self.reportsServer, null);
        }
        catch (e) {
            AntLib.toLog('AntRPC.download: ' + e);
        }        
    },
    download : function( flvlink ) {
        
        if ( AntLib.inPrivate )
            return;
        
        var self = AntRPC;
        var addonList;
        
        if ( AntLib.getFirefoxVersion() < 4 ) {
            
            self.downloadCallback( flvlink );
        }
        else {
            Components.utils.import("resource://gre/modules/AddonManager.jsm");
            AddonManager.getAllAddons( function(addonList){
                                        self.downloadCallback( flvlink, addonList );
                                       } );
        }
    },
    
    heartBeat : function() {
        
        try {
            
            if ( AntLib.inPrivate )
                return;
            
            if ( !AntPrefs.canSendHeartBeat )
                return;
            
            
            var self = AntRPC;
            var request =  {
                "version":  "1.0",
                "id":       1,
                "method":   "heartbeat",
                "params":
                [
                    "ant.report.addon.heartbeat",
                    {
                        'uid':          AntRank.UUID,
                        'uagent':       AntPrefs.getUserAgent(),
                        'locale':       AntPrefs.getAcceptLang(),
                        'install_ts':   AntPrefs.installTs,
                        'default_se':   AntPrefs.defaultSe,
                        'ar_enabled':   AntPrefs.isRankMode,
                        'vr_enabled':   AntPrefs.isVideorepportsOn,
                        'timezone':     AntLib.getTimezone(),
                        'display_mode': (AntPrefs.displayMode == 'addonsbar') ? "statusbar" : AntPrefs.displayMode
                    }
                ],
                'agent':    AntBar.getAgent()
            };
            AntLib.makeRequest( request, self.reportsServer, null );
        }
        catch (e) {
            AntLib.toLog( 'AntRPC.heartBeat: ' + e );
        }
    },
    
    detectedVideos : function(doc) {
        
        try {
            
            if ( AntLib.inPrivate )
                return;
            
            if ( !AntPrefs.canSendStats ) {
              return;
            }
            
            var self = AntRPC;
            var data = AntTabMan.getAntData(doc);
            
            var antflv = data.videos;
            if (!antflv || antflv.length == 0){
                return;
            }
            
            var report = {
                "version":  "1.0",
                "id":       1,
                "method":   "log",
                "params":
                [
                    "ant.report.addon.detected",
                    {
                        "url":      encodeURI(doc.URL),
                        "ref":      encodeURI(doc.referrer),
                        "streams":  [],
                        "uid":      AntRank.UUID,
                        "uagent":   AntPrefs.getUserAgent(),
                        "top":      data.totalTime(),
                        "atop":     data.totalActiveTime()                        
                    }
                ],
                "agent":    AntBar.getAgent()
            };
            
            for ( var i = 0; i < antflv.length; i++ ) {
                report.params[1].streams.push(
                    {"stream":encodeURI(antflv[i].getUri().spec),
                     "type":antflv[i].getContentType(),
                     "length":parseInt(antflv[i].getContentLength())
                    });
            }

            AntLib.makeRequest( report, self.reportsServer, null );
        }
        catch (e) {
            AntLib.toLog( 'AntRPC.detectedVideos: ' + e );
        }
    }
};
